import "dart:html";
import "dart:core";
import "day.dart";

class Calendar
{ 
  Calendar()
  {  
    _days = new Map();
    DateTime now = new DateTime.now(); 
    int weekday = now.weekday;
    
    for (int i = 1; i < 8; i++)
    {
      _currentWeek.add(now.add(new Duration(days:i-weekday)));
      
    }
    _yearHeader = querySelector("#year");
    _weekHeader = querySelector("#week");
    _divWeekdays.add(querySelector("#monday"));
    _divWeekdays.add(querySelector("#tuesday"));
    _divWeekdays.add(querySelector("#wednesday"));
    _divWeekdays.add(querySelector("#thursday"));
    _divWeekdays.add(querySelector("#friday"));
    _divWeekdays.add(querySelector("#saturday"));
    _divWeekdays.add(querySelector("#sunday"));
    for (int i = 0; i < 7; i++)
    {
      _divWeekdays[i].querySelector(".dayofweek").innerHtml = _weekdays[i];
    }    
    _next = querySelector("#next");
    _previous = querySelector("#previous");
    
    render(0);
    _next.onClick.listen((_) => render(1));
    _previous.onClick.listen((_) => render(-1));
  }
     
  void render(int week_offset) 
  {
    _cWeekOffset += week_offset;
    
    /// Update current week
    for (int i = 0; i <_currentWeek.length; i++)
    {
      _currentWeek[i] =_currentWeek[i].add(new Duration(days:7*week_offset));
    }

    for (int i = 0; i < 7; i++)
    {
      _divWeekdays[i].querySelector(".date").innerHtml = _currentWeek[i].day.toString();
      _divWeekdays[i].querySelector(".month").innerHtml = _months[_currentWeek[i].month - 1];
      _divWeekdays[i].querySelector(".content").children.clear();
      DateTime date = _currentWeek[i];
      if (_days.containsKey(date.year))
      {        
        if (_days[date.year].containsKey(date.month))
        {
          Map<int, Day> week = _days[date.year][date.month];
          if (week.containsKey(date.day))
          {
            List<Element> children = _divWeekdays[i].querySelector(".content").children;
            children.add(week[_currentWeek[i].day].content);
          }
        }
      }
    }
    _weekHeader.innerHtml = calcWeekOfYear(_cWeekOffset).toString();
  }
  
  int calcWeekOfYear(int week_offset)
  {
    DateTime current = new DateTime.now().add(new Duration(days:7*week_offset));
    _yearHeader.innerHtml = current.year.toString();
    /// First day of year is not always monday! But we assume every week is 7 days. Offset the first week by number of days since monday.
    DateTime firstDayOfYear = new DateTime(current.year);     
    _daysPerMonth[1] = 31 + firstDayOfYear.weekday;
    
    /// Leap year
    _daysPerMonth[2] = (current.year % 4 == 0 && current.year % 100 == 0 && current.year % 400 == 0) ? 29 : 28;
    _daysPerMonth[3] = 31;
    _daysPerMonth[4] = 30;
    _daysPerMonth[5] = 31;
    _daysPerMonth[6] = 30;
    _daysPerMonth[7] = 31;
    _daysPerMonth[8] = 31;
    _daysPerMonth[9] = 30;
    _daysPerMonth[10] = 31;
    _daysPerMonth[11] = 30;
    _daysPerMonth[12] = 31;
    
    int currentDayOfYear = 0;
    
    for (int i = 1; i < current.month; i++)
    {
      currentDayOfYear += _daysPerMonth[i];
    }
    currentDayOfYear += current.day;
    return (currentDayOfYear.toDouble() / 7).ceil();
  }
  
  Day getDay(int year, int month, int day)
  {
    //must check if correct date before this  
     
    DateTime checkDate = new DateTime(year,month,day);
    if (checkDate.day != day || checkDate.year != year || checkDate.month != month) throw(new StateError("Date is not valid"));
      
     if(!_days.containsKey(year))
     {
       _days[year] = new Map();
     }
     
     if( !_days[year].containsKey(month))
     {
       _days[year][month] = new Map();
     }

     if( !_days[year][month].containsKey(day))
     {
       _days[year][month][day] = new Day();
     }    
     
     return  _days[year][month][day];
  }   
  List<DateTime> _currentWeek = new List();
  List<DivElement> _divWeekdays = new List();
  HeadingElement _yearHeader;
  HeadingElement _weekHeader;
  List<String> _weekdays = ["mon","tue","wed","thu","fri","sat","sun"];
  List<String> _months = ["januari","february","march","april","may","june","july","august","september","october","november","december"];
  Map<int, int> _daysPerMonth = new Map(); 
  ButtonElement _next;
  ButtonElement _previous;
  int _cWeekOffset = 0;
  Map<int, Map<int, Map<int, Day>>> _days; 
}